# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this file,
# You can obtain one at http://mozilla.org/MPL/2.0/.
"""

Error class definitions for the Sauropod data store.

"""


class Error(Exception):
    """Base error type for pysauropod."""
    pass


class AuthenticationError(Error):
    """Error raised when invalid key or credentials are provided."""
    pass


class PermissionError(Error):
    """Error raised when accessing data without appropriate permissions."""
    pass


class SessionExpiredError(Error):
    """Error raised when attempting to use an expired session."""
    pass


class ConflictError(Error):
    """Error raised when a write fails due to conflicting versions."""
    pass


class ConnectionError(Error):
    """Error raised when there is a problem connecting to the server."""
    pass


class ServerError(Error):
    """Base class for unexpected errors generated by the server."""

    def __init__(self, message="", status_code=500):
        super(ServerError, self).__init__(message)
        self.status_code = status_code


class ServerBusyError(Error):
    """Error raised when the server is busy and wants you to back off."""

    def __init__(self, message="", status_code=503, retry_after=None):
        super(ServerBusyError, self).__init__(message, status_code)
        self.retry_after = retry_after
